@@ -1562,6 +1562,23 @@ var PlacesUtils = {
     stream.close();
   },
 
+  /**
+   * Creates a filename for bookmarks backup files.
+   *
+   * @param [optional] aDateObj Date object used to build the filename.
+   *                            Will use current date if empty.
+   * @return A bookmarks backup filename.
+   */
+  getBackupFilename:
+  function PU_getBackupFilename(aDateObj) {
+    if (!aDateObj)
+      aDateObj = new Date();
+    // Use YYYY-MM-DD (ISO 8601) as it doesn't contain illegal characters
+    // and makes the alphabetical order of multiple backup files more useful.
+    var date = aDateObj.toLocaleFormat("%Y-%m-%d");
+    return "bookmarks-" + date + ".json";
+  },
+
   /**
    * ArchiveBookmarksFile()
    *
@@ -1586,12 +1603,9 @@ var PlacesUtils = {
         return; // unable to create directory!
     }
 
-    // construct the new leafname
-    // Use YYYY-MM-DD (ISO 8601) as it doesn't contain illegal characters
-    // and makes the alphabetical order of multiple backup files more useful.
-    var date = new Date().toLocaleFormat("%Y-%m-%d");
-    var backupFilename = "bookmarks-" + date + ".json";
-
+    // Construct the new leafname.
+    var date = new Date();
+    var backupFilename = this.getBackupFilename(date);
     var backupFile = null;
     if (!aForceArchive) {
       var backupFileNames = [];
@@ -1601,7 +1615,7 @@ var PlacesUtils = {
       // old backups with a localized name (bug 445704).
       var localizedFilename = this.getFormattedString("bookmarksArchiveFilename", [date]);
       var localizedFilenamePrefix = localizedFilename.substr(0, localizedFilename.indexOf("-"));
-      var rx = new RegExp("^(bookmarks|" + localizedFilenamePrefix + ")-.+\.json");
+      var rx = new RegExp("^(bookmarks|" + localizedFilenamePrefix + ")-([0-9-]+)\.(json|html)");
 
       var entries = bookmarksBackupDir.directoryEntries;
       while (entries.hasMoreElements()) {
@@ -1609,10 +1623,11 @@ var PlacesUtils = {
         var backupName = entry.leafName;
         // A valid backup is any file that matches either the localized or
         // not-localized filename (bug 445704).
-        if (backupName.match(rx)) {
+        var matches = backupName.match(rx);
+        if (matches) {
           if (backupName == backupFilename)
             backupFile = entry;
-          backupFileNames.push(backupName);
+          backupFileNames.push({ filename: backupName, date: matches[2] });
         }
       }
 
@@ -1626,11 +1641,12 @@ var PlacesUtils = {
         // number specified in the pref.
         if (!backupFile)
           numberOfBackupsToDelete++;
-
-        backupFileNames.sort();
+        backupFileNames.sort(function compare(a, b) {
+          return a.date < b.date ? -1 : a.date > b.date ? 1 : 0;
+        });
         while (numberOfBackupsToDelete--) {
           let backupFile = bookmarksBackupDir.clone();
-          backupFile.append(backupFileNames[0]);
+          backupFile.append(backupFileNames[0].filename);
           backupFile.remove(false);
           backupFileNames.shift();
         }
